/**
 * Node application for mkgichessclub
 */
var ChessServer = require('./chessserver'),
    util = require('util'),
    config = require('./config');

var io = require('socket.io').listen(config.server_port),
    chess_server = ChessServer.create(io, config),
    check_online_users = function () {
        chess_server.get_online_players();
        setTimeout(check_online_users, config.online_player_refresh_interval);
    },
    // push online player to clients, burden presence server implementation
    timer_online = setTimeout(check_online_users, config.online_player_refresh_interval);

io.sockets.on('connection', function (socket) {

    /**
     * setting session variables from real time front controller
     * only received when starting a real time game from chessrt.php
     */
    socket.on('set session', function (session) {
        chess_server.join_player(socket, session);
    });

    /**
     * Event emitted when player make a move on chess board. Note that
     * the timer data will be removed as soon as possible.
     *
     * @param data Object with move data. It has this:
     *      move_str:"Pe2-e4",
     *      move_obj:{"src":12,"dest":28,"kill":false},
     *      gameid:5,
     *      player:"davidmpaz",
     *      chess_move:{"from":"e2","to":"e4"},
     *      promotion:false,
     *      own_time:300000,
     *      opp_time:294000
     */
    socket.on('player move', function (data, fn) {
        chess_server.handle_move(socket, data, fn);
    });

    /**
     * Event emitted by player when timer timeout
     *
     * @param data Object It contains:
     *      move_str: 'resign',
     *      gameid: [game identifier],
     *      player: [player identifier],
     *      opp_timer: 24000,
     *      own_timer: 30000
     * @param callback Function to call
     */
    socket.on('game over timeout', function(data, callback){
        chess_server.handle_timeout(socket, data, callback);
    });

    /**
     * Event emitted by clients inviting opponent player to go to real time mode
     *
     * @param data object containing:
     *                  player: opponent player identifier,
     *                  inviter: current player identifier,
     *                  time: time limit for the match,
     *                  game: game id
     * @param fn callback to execute
     */
    socket.on('go real time', function (data, fn) {
       chess_server.invite_player(socket, data.player, data.game, data.time, fn);
    });

    /**
     * Event emitted from clients requesting to sync opponent timer
     *
     * @param data Object with gameid and player color
     * @param fn callback to call
     */
    socket.on('sync', function (data, fn) {
        chess_server.sync_players(socket, data, fn);
    });

    /**
     * on player connect store socket for later user this event is fired
     * from pages different than real time  front controller.
     *
     * @param player player identifier on mkgichessclub
     */
    socket.on('player connected', function (player) {
        chess_server.connect_player(socket, {player: player});
    });

    /**
     * handle disconnection
     */
    socket.on('disconnect', function (data) {
        socket.get('player', function (err, player) {
            if(player === null) {
                // nothing to do
                util.debug('disconecting user without session.');
                return;
            }

            chess_server.notify_player_disconnection(socket, player);
        });
    });

});