/**
 * Class for a chess client
 *
 * This interacts with the nodejs server, store a list of moves and update board
 * as required for the MKGIChess application to properly work with its UI.
 *
 * @param player_color
 * @param config
 * @param player
 * @param move_set
 * @param socket
 * @param gameid
 */
function ChessClient(socket, player, player_color, gameid, move_set, config) {

    if (this instanceof ChessClient) {
        // Chess instance to validate moves
        this.chess = new Chess();
        // socket to use to emit events to server
        this.socket = socket;
        // White player identifier
        this.player = player;
        // player color,can be "White" or "Black"
        this.player_color = player_color;
        // Game identifier on database
        this.gameid = gameid;
        // Array of moves
        this.moves = move_set;
        // configuration for the nodejs/php server and what would be needed
        this.config = config;
        // board positions
        this.board = {};
        // can make move (it is his turn)
        this.can_move = false;
        // game end by time limit
        this.timeout = false;
        // here also goes the timers, left 'undefined' on purpose
    } else {
        return new ChessClient(
            socket, player, player_color, gameid, move_set, config);
    }
}

/**
 * Initialize the client
 */
ChessClient.prototype.init = function init() {

    // load the game from pgn exported by server
    var self = this;
    $.get(
        this.config.server_url + '/pgnformat.php?gameid=' + this.gameid,
        function(pgn){
            self.chess.load_pgn(pgn);
        }
    );

    this.timeout = (this.config.opp_timer === 0 || this.config.own_timer === 0);

    // send session data
    this.socket.emit('set session', {game: this.gameid,
        color: this.player_color, player: this.player,
        invited: this.config.invited, time: this.config.time});

    // going real time?
    this.create_timers();
    // init the board
    this.build_board();
    // can player move
    this.update_player_turn();
    this.check_state();
};

ChessClient.prototype.game_over_timeout = function game_over_timeout() {
    // loose by time out, send a resignation ?
    // TODO change this to specify player lost by time
    var self = this,
        opp_timer = this.opponent_timer.getTimerValue(),
        own_timer = this.own_timer.getTimerValue(),
        // here player is the one sending message, not necessary the one who
        // lost by running out of time
        data = {move_str:'resign', gameid: this.gameid, player: this.player,
            opp_timer: opp_timer, own_timer: own_timer};

    this.timeout = true;

    this.socket.emit('game over timeout', data, function(ack){
        if(ack == 'ok') {
            // update UI
            self.check_state();
            // make the call on aknowledgment
            if(own_timer === 0) {
                // make the call on aknowledgment
                self.update_server_move(data);
            }
        } else {
            // is somebody hacking ?
            alert('Timer error. You are reporting a game over by timeout but ' +
                'other player claim it is not true. ' + ack);
        }
    });
};

ChessClient.prototype.switch_timers = function switch_timers(data) {
    var self = this;

    // if no timer nothing to switch
    if(this.config.time_limit === 0) return;

    // this is called when the player already moved
    if( this.can_move ) {
        if(typeof data != 'undefined') {
            this.own_timer.resetTimer({autostart: true,
                time_in_seconds: Math.floor(data.opp_time / 1000),
                buzzer: function(timer){
                    self.game_over_timeout();
                }
            });
        }
        else {
            this.own_timer.startTimer({
                buzzer: function(timer){
                    self.game_over_timeout();
                }
            });
        }
        this.opponent_timer.pauseTimer();
    } else {
        if(typeof data != 'undefined') {
            this.opponent_timer.startTimer({
                buzzer: function(timer){
                    self.game_over_timeout();
                }
            });
        } else {
            this.opponent_timer.startTimer({
                buzzer: function(timer){
                    self.game_over_timeout();
                }
            });
        }
        this.own_timer.pauseTimer();
    }
};

ChessClient.prototype.build_board = function build_board() {

    var x, y, tmp,
        isw = false,
        index = 0;
    for (y = 1; y < 9; y += 1) {
        for (x = 1; x < 9; x += 1) {
            tmp = $('td#xb' + index + ' div');

            isw = (typeof tmp === 'jQuery' && tmp.length > 0 &&
                $(tmp).children('img').attr('alt').indexOf('White') !== -1);

            this.board[x + ',' + y] = {
                position: index,
                parent_td_id: 'xb' + index,
                child_div: tmp,
                is_white_piece: isw
            };
            index += 1;
        }
    }
};

/**
 * Build a move object to use it with chess engine.
 * Need to take care here of promotions, en-passant and castling. Also will
 * do extra UI updates for special moves.
 *
 * The callback accept two parameters, the move to be consumed by the chess
 * engine and if the move was a promotion, the piece to where it was promoted.
 */
ChessClient.prototype.get_chess_move = function get_chess_move(move, callback) {
    var piece = 'Q',
        promob = move.charAt(0) == 'P' && move.charAt(5) == '1',
        promow = move.charAt(0) == 'P' && move.charAt(5) == '8',
        castlingb = move.charAt(0) == 'K' && move.slice(1,3) == 'e8',
        castlingw = move.charAt(0) == 'K' && move.slice(1,3) == 'e1';


    // check if castling (black)
    if(castlingb){
        if(move.slice(4,6) == 'g8'){
            this.update_board_ui(this.get_move('Rh8-f8'), 'Rh8-f8');
            callback('O-O', false);
        }

        if(move.slice(4,6) == 'c8'){
            this.update_board_ui(this.get_move('Ra8-d8'), 'Ra8-d8');
            callback('O-O-O', false);
        }
    }

    // check if castling (white)
    if(castlingw){
        if(move.slice(4,6) == 'g1'){
            this.update_board_ui(this.get_move('Rh1-f1'), 'Rh1-f1');
            callback('O-O', false);
        }

        if(move.slice(4,6) == 'c1'){
            this.update_board_ui(this.get_move('Ra1-d1'), 'Ra1-d1');
            callback('O-O-O', false);
        }
    }

    // check promotion black player
    if(promob && this.can_move) {
        $('#promotion-dialog').dialog({
            title: 'Promote to a greater chessman !!',
            height: 250,
            width: 350,
            modal: true,
            resizable: false,
            buttons: {
                Ok: function() {
                    piece = $('input:radio:checked').attr('value');
                    $( this ).dialog( "close" );
                },
                Cancel: function() {
                    $( this ).dialog( "close" );
                }
            },
            close: function() {
                callback({from: move.slice(1,3), to: move.slice(4,6), promotion: piece.toLowerCase()}, piece);
            }
        });
    }

    // check promotion white player
    if(promow && this.can_move) {
        $('#promotion-dialog').dialog({
            title: 'Promote to a greater chessman !!',
            height: 250,
            width: 350,
            modal: true,
            resizable: false,
            buttons: {
                Ok: function() {
                    piece = $('input:radio:checked').attr('value');
                    $( this ).dialog( "close" );
                },
                Cancel: function() {
                    $( this ).dialog( "close" );
                }
            },
            close: function() {
                callback({from: move.slice(1,3), to: move.slice(4,6), promotion: piece.toLowerCase()}, piece);
            }
        });
    }

    // if we reach here just return result
    if(! (promob || promow || castlingb || castlingw) ) {
        callback({from: move.slice(1,3), to: move.slice(4,6)}, false);
    }
};

/**
 * Public API. Handle a current player move, should deactivate move handlers until next
 * player's turn, update board UI and emit the "player move" event to nodejs
 * server with built move data needed ("kill" property is calculated on server)
 * by opponent player to update his board UI.
 *
 * Data is a "move" object with src, dest and kill numbers
 *
 * @param {Object} move     Object with source, destination and killing data
 *                          as required for MKGIChess client to do its job.
 */
ChessClient.prototype.move = function move (move_str, callback) {
    // validate move
    var self = this;
    this.get_chess_move(move_str, function(mo, promotion){

        if(self.chess.move(mo) ===null) {
            // not valid move
            if(typeof callback === 'function') { callback(false); }
        } else {
            // get move object information
            var m = self.get_move(move_str);
            // clear the old move path
            self.clear_move_path();
            // update move list
            self.update_move_list(m);
            // draw path again
            self.draw_move_path();
            // adjust UI after the move, promotion can be:
            // Q, R, B, K, q, r, b, k, false
            self.update_ui(m, move_str, promotion);
            // lets update the player turn
            self.can_move = ! self.can_move;
            // emit "player move" event to nodejs server with move data
            var data = {move_str: move_str, move_obj: m, gameid: self.gameid,
                    player: self.player, chess_move: mo, promotion: promotion,
                    own_time: (self.config.time_limit) ? self.own_timer.getTimerValue() : 0,
                    opp_time: (self.config.time_limit) ? self.opponent_timer.getTimerValue() : 0};
            self.socket.emit("player move", data, function(ack){
                // pause current player clock and resume opponent
                self.switch_timers();
                // only one end can update server with moves
                if(self.config.invited) {
                    // make the call on aknowledgment
                    self.update_server_move(data);
                }
            });
            // rebuild the board
            self.build_board();
            // return needed result
            if(typeof callback == 'function') { callback(true); }
        }
    });
};

/**
 * Handle a opponent player move, should activate move handlers until next
 * player's turn, update board UI.
 *
 * Data is an object with move (src, dest and kill numbers) also the game id and
 * the move as String.
 *
 * @param Number game       The game identifier
 * @param {Object} data     Object with source, destination and killing data
 *                          as required for MKGIChess client to do its job.
 */
ChessClient.prototype.move_opponent = function move_opponent (data, callback) {
    var self = this;

    this.get_chess_move(data.move_str, function(mo, promotion){
        // this takes care of UI for castling in opponent side, thats why
        // we need to call it
    });

    if(self.chess.move(data.chess_move) === null) {
        // not valid move
        console.log('Invalid move. Your opponent is probably trying to cheat you.');
        if(typeof callback == 'function') { callback(false); }
    } else {
        // get move object information
        var m = data.move_obj;
        // clear the old move path
        this.clear_move_path();
        // update move list
        this.update_move_list(m);
        // draw path again
        this.draw_move_path();
        // adjust UI after the move
        this.update_ui(m, data.move_str, data.promotion);
        // lets update the player turn
        this.can_move = ! this.can_move;
        // start own clock and pause opponent one
        client.switch_timers(data);
        // rebuild the board
        this.build_board();
        // return needed result
        if(typeof callback == 'function') { callback(true); }
    }
};

ChessClient.prototype.update_server_move = function update_server_move (data) {
    // handle the move, store in db, etc... through queued ajax call
    var post = {chessmove: data.move_str, player: data.player};

    if(this.player == data.player) {
        post.mvtime = data.own_time;
    } else {
        post.mvtime = data.opp_time;
    }

    var self = this;
    $.ajaxq('mkgichess-queue',{
        type: 'POST',
        url: this.config.server_url + '/chessajaxied.php?gameid=' + data.gameid,
        data: post,
        success: function (body) {
            //self.socket.emit('update move description', body);
        }
    });
};

ChessClient.prototype.update_move_list = function update_move_list (move) {

    var index = this.moves.length;

    index += 1;
    this.moves[index] = move.src;
    index += 1;
    this.moves[index] = move.dest;
    index += 1;
    this.moves[index] = move.kill;
};

/**
 * Updates the can_move object property
 */
ChessClient.prototype.update_player_turn = function update_player_turn () {

    // is player turn, get move count, then check
    var movecount = Math.floor(this.moves.length / 3);
    if (movecount > 0) {
        if (movecount % 2 === 0) {
            this.can_move = this.player_color == 'white';
        }else {
            this.can_move = this.player_color == 'black';
        }
    } else {
        this.can_move = this.player_color == 'white';
    }
};

/**
 * Taken from chess.php
 *
 * This draw a path to get a clue of what move was made.
 */
ChessClient.prototype.draw_move_path = function draw_move_path () {

    var lastmove_offset = this.moves.length - 3;
    var movecount = Math.floor(this.moves.length / 3);

    if ( movecount > 0 ) {
        if ( movecount % 2 === 0 ) {
            renderMove(-1, -1,
                this.moves[lastmove_offset], this.moves[lastmove_offset + 1],
                    false);
        }
        else {
            renderMove(this.moves[lastmove_offset], this.moves[lastmove_offset + 1],
                    -1, -1, false);
        }
    }
};

/**
 * This make the oppsite of renderMove original API, it clear the path so we
 * can render other moves.
 */
ChessClient.prototype.clear_move_path = function clear_move_path () {

    var i, lx, ly, iswhitecell, classname, cell;

    for ( i = 0; i < 64; i++ ) {
        lx = Math.floor(i % 8);
        ly = Math.floor(i / 8);

        iswhitecell = ( (lx + ly%2)%2 == 1 );

        classname = 'b_std';

        if ( iswhitecell ) {
            classname += '_white';
        } else {
            classname += '_black';
        }

        cell = document.getElementById('xb' + i);
        cell.setAttribute('class', classname);
        cell.setAttribute('className', classname);
    }
};

/**
 * Given the move string like Ph2-h4, calculate the index of tiles src and dest.
 * And built the move object to send to server.
 *
 * @return Array        The src and dest values
 * @param move String   The move made by player
 */
ChessClient.prototype.get_move = function get_move (move) {

    if (6 > move.length) return false;
    //build the move
    var src = move.slice(1, 3),
        dest = move.slice(4, 6),
        kill = move.charAt(3),

        src_y = Number(src.charAt(1)),
        dest_y = Number(dest.charAt(1)),
        // get the cero based char index
        src_x = src.charCodeAt(0) - 96,
        dest_x = dest.charCodeAt(0) - 96;

    // rigth now we are not calculating the kill value, we just
    // state that a piece has been taken.
    return {src: this.board[src_x + ',' + src_y].position,
        dest: this.board[dest_x + ',' + dest_y].position,
        kill: kill === 'x'};
};

//////////////////// User Iterface (UI) Features \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
ChessClient.prototype.check_state = function check_state () {
    var title = '',
        gameover = function (title){
            $('#gameover-dialog').dialog({
                title: title,
                height: 200,
                width: 350,
                modal: true,
                resizable: false,
                show: {
                    effect: "pulsate",
                    duration: 1000
                },
                hide: {
                    effect: "fade",
                    duration: 1000
                },
                buttons: {
                    Ok: function() {
                        $( this ).dialog( "close" );
                    }
                },
                close: function() {
                    var url = 'http://' + window.location.host +
                        '/chess.php' +
                        window.location.search;
                    window.location.href = url;
                }
            });
        };

    // add small effects
    if(this.chess.in_check() && !this.can_move) {
        $('img[id^="img-K"]').effect('shake', {distance: 5});
    }

    if (this.chess.game_over()) {
        if(this.chess.in_checkmate()){
            if (! this.can_move) {
                // current player loose
                title = 'You have been checkmated !!';
            } else {
                title = 'You win by checkmate !!';
            }
        }

        if(this.chess.in_stalemate()){
            title = 'The game is draw by stalemate.';
        }

        if(this.chess.in_stalemate()){
            title = 'The game is draw by threefold repetition.';
        }

        if(this.chess.insufficient_material()){
            title = 'The game is draw by insufficient material.';
        }

        if(this.chess.in_draw()){
            title = 'The game is draw.';
        }

        gameover( title );
    // check if game is over by timeout
    } else if(this.timeout){
        if (this.own_timer.getTimerValue() === 0) {
            // current player loose
            title = 'You ran out of time. You lost !!';
        } else {
            title = 'Your opponent ran out of time. You win !!';
        }

        gameover( title );

    }
};

/**
 *
 */
ChessClient.prototype.update_ui = function update_ui (move_obj, move_str, promo) {

    this.update_board_ui(move_obj, move_str, promo);
    this.update_move_list_ui();

    // update move box
    $('#cmdform input[name="chessmove"]').value = (move_str);
    // turn indicator
    if(this.can_move) {
        $('p.warning').html("It is your opponent turn. (You must wait.)");
    } else {
        $('p.warning').html("");
    }

    // check state of the game
    this.check_state();
};

/**
 * This is in charge to update the chess board UI, updates DIVs and IMGs with
 * proper html id and handlers to work fully integrated with MKGIChess. All the
 * work id done starting from source and destination position of the move. Must
 * take care of both side, the current player and the opponent making their moves
 * as such need to check when we are taking or just moving a piece from both point
 * of view and update the HTML accordingly.
 *
 * The order of sentences matter!!
 *
 */
ChessClient.prototype.update_board_ui = function update_board_ui (move_obj, move_str, promo) {

    // get html ids (old and new destinations and source)
    var dest_old_id = String($('#xb' + move_obj.dest + ' div').attr('id')),
        src_old_id = String($('#xb' + move_obj.src + ' div').attr('id')),
        src_new_id = String('-' + src_old_id.slice(1)),
        src = $('#xb' + move_obj.src + ' div img').attr('src'),
        alt = $('#xb' + move_obj.src + ' div img').attr('alt'),
        dest_new_id = '',
        piece_name = {Q : 'Queen', K: 'King', R: 'Rook', B: 'Bishop', N: 'Knight'},
        tmp_alt = '', tmp_src = '', tmp_color = '';

    // is current player moving
    if(this.can_move) {
        if(move_obj.kill) { // the move is for killing
            // replace images src, id and alt attributes, check if promotion
            if(typeof promo == 'undefined' || ! promo){
                // normal killing move, no promotion
                $('#xb' + move_obj.dest + ' div>img').attr('src', src);
                $('#xb' + move_obj.dest + ' div>img').attr('alt', alt);
                dest_new_id = String(move_str.charAt(0) + dest_old_id.slice(1));
            } else {
                // replace target with selected promoted piece
                tmp_alt = this.player_color + ' ' + piece_name[promo],
                tmp_src = 'images/wcg/' +
                    this.player_color.charAt(0).toLowerCase()+
                    piece_name[promo].toLowerCase() + '.png';
                $('#xb' + move_obj.dest + ' div>img').attr('src', tmp_src);
                $('#xb' + move_obj.dest + ' div>img').attr('alt', tmp_alt);
                dest_new_id = String(promo.toUpperCase() + dest_old_id.slice(1));
            }
        } else { // not killing move
            // replace images src, id and alt attributes, check if promotion
            if(typeof promo == 'undefined' || ! promo){
                // normal move, not killing neither promotion
                $('#xb' + move_obj.dest + ' div>img').attr('src', src);
                $('#xb' + move_obj.dest + ' div>img').attr('alt', alt);
                dest_new_id = String(move_str.charAt(0) + dest_old_id.slice(1));
            } else {
                // normal move with promotion, replace target with selected piece
                tmp_alt = this.player_color + ' ' + piece_name[promo],
                tmp_src = 'images/wcg/' +
                    this.player_color.charAt(0).toLowerCase() +
                    piece_name[promo].toLowerCase() + '.png';
                $('#xb' + move_obj.dest + ' div>img').attr('src', tmp_src);
                $('#xb' + move_obj.dest + ' div>img').attr('alt', tmp_alt);
                dest_new_id = String('x' + dest_old_id.slice(1));
            }
        }

    } else { // opponent is moving, mostly the same but for an opponent move
        if(move_obj.kill) {
            // replace images src, id and alt attributes, check if promotion
            if(typeof promo == 'undefined' || ! promo){
                // normal move
                $('#xb' + move_obj.dest + ' div>img').attr('src', src);
                $('#xb' + move_obj.dest + ' div>img').attr('alt', alt);
            } else {
                // replace target with selected piece
                if(this.player_color == 'White') {
                    tmp_color = 'Black';
                } else {
                    tmp_color = 'White';
                }
                tmp_alt = tmp_color + ' ' + piece_name[promo],
                tmp_src = 'images/wcg/' +
                    tmp_color.charAt(0).toLowerCase() +
                    piece_name[promo].toLowerCase() + '.png';
                $('#xb' + move_obj.dest + ' div>img').attr('src', tmp_src);
                $('#xb' + move_obj.dest + ' div>img').attr('alt', tmp_alt);
            }

            dest_new_id = String('x' + dest_old_id.slice(1));
        } else {
            // replace images src, id and alt attributes, check if promotion
            if(typeof promo == 'undefined' || ! promo){
                // normal move
                $('#xb' + move_obj.dest + ' div>img').attr('src', src);
                $('#xb' + move_obj.dest + ' div>img').attr('alt', alt);
            } else {
                // replace target with selected piece
                if(this.player_color == 'White') {
                    tmp_color = 'Black';
                } else {
                    tmp_color = 'White';
                }
                tmp_alt = tmp_color + ' ' + piece_name[promo],
                tmp_src = 'images/wcg/' +
                tmp_color.charAt(0).toLowerCase() +
                piece_name[promo].toLowerCase() + '.png';
                $('#xb' + move_obj.dest + ' div>img').attr('src', tmp_src);
                $('#xb' + move_obj.dest + ' div>img').attr('alt', tmp_alt);
            }

            dest_new_id = String('x' + dest_old_id.slice(1));
        }
    }

    // replace images src, id and alt attributes, check if promotion
    $('#xb' + move_obj.dest + ' div>img').attr('id', 'img-' + dest_new_id);
    $('#xb' + move_obj.src + ' div>img').attr('alt', 'Empty');
    $('#xb' + move_obj.src + ' div>img').attr('src', 'images/wcg/empty.png');
    $('#xb' + move_obj.src + ' div>img').attr('id', 'img-' + src_new_id);
    // cursor style modified by mouse handlers
    document.getElementById('img-' + src_new_id).style.cursor = 'default';

    // replace the ids, cursor and handlers of parent div
    $('#xb' + move_obj.src + ' div').attr('id', src_new_id);
    $('#xb' + move_obj.dest + ' div').attr('id', dest_new_id);

    // need to split here in two: current user is moving or opponent is moving
    // then we need to specify if the move is taken a piece or not

    // avoid registering a source piece again
    $('#xb' + move_obj.src + ' div img').removeAttr('onload');

    if(this.can_move) {
        if(move_obj.kill) {
            dnd_replaceTarget(document.getElementById(
                '#xb' + move_obj.src), src_old_id, src_new_id);
            dnd_replaceTarget(document.getElementById(
                '#xb' + move_obj.dest), dest_old_id, dest_new_id);
            // current player is moving and killing, deregister the killed  piece
            dnd_deregisterSource(src_old_id);
            $('#img-' + dest_new_id)
                .attr('onload', "dnd_registerSource(this, \"" + dest_new_id + "\")");
        } else {
            dnd_replaceTarget(document.getElementById(
                '#xb' + move_obj.src), src_old_id, src_new_id);
            dnd_replaceTarget(document.getElementById(
                '#xb' + move_obj.dest), dest_old_id, dest_new_id);
            // current player is moving, deregister the piece and register new one
            dnd_deregisterSource(src_old_id);
            $('#img-' + dest_new_id)
                .attr('onload', "dnd_registerSource(this, \"" + dest_new_id + "\")");
        }
        document.getElementById('img-' + dest_new_id).style.cursor = 'pointer';
        // reset style for src image, in case dragndrop executed
        $('#img-' + src_new_id).removeAttr('style');
    }
    else { // opponent move
        if(move_obj.kill) {
            $('#xb' + move_obj.dest + ' div img').removeAttr('onload');
            dnd_replaceTarget(document.getElementById(
                '#xb' + move_obj.src), src_old_id, src_new_id);
            dnd_replaceTarget(document.getElementById(
                '#xb' + move_obj.dest), dest_old_id, dest_new_id);
            dnd_deregisterSource(dest_old_id);
            $('#img-' + dest_new_id).effect('bounce');
        } else {
            dnd_replaceTarget(document.getElementById(
                '#xb' + move_obj.src), src_old_id, src_new_id);
            dnd_replaceTarget(document.getElementById(
                '#xb' + move_obj.dest), dest_old_id, dest_new_id);
            $('#img-' + dest_new_id).effect('highlight');
        }
        document.getElementById('img-' + dest_new_id).style.cursor = 'default';
    }

    // replace the ids, cursor and handlers of parent div
    // invoke this after replacing targets
    $('#xb' + move_obj.src + ' div')
        .attr('onclick', "clickHandler(\"" + src_new_id +"\")");
    // source tile need cursor change to its default
    document.getElementById(src_new_id).style.cursor = 'default';
    $('#xb' + move_obj.dest + ' div')
        .attr('onclick', "clickHandler(\"" + dest_new_id +"\")");

    // reset local UI var
    dnd_dragPiece   = null;
    dnd_mouseOffset = null;
    dnd_originalPos = null;
    dnd_lastsourcedata = null;
    moveparts = [];
};

/**
 *
 */
ChessClient.prototype.update_move_list_ui = function update_move_list_ui () {
    var history = this.chess.history(),
        html = [],
        length = history.length,
        i = 0, move = 0,
        css = '';

    for(i; i < length; i +=2) {
        if( (Math.floor(i / 2) % 2 ) === 0) {
            css = 'odd';
        } else {
            css = 'even';
        }

        html.push('<tr><td class="movehistory_count_' + css + '">');
        html.push(move += 1);
        html.push('. </td><td class="movehistory_move_'+ css + '">&nbsp;');
        html.push(history[i]);
        html.push('</td>');

        if (i < length - 1) {
            html.push('<td class="movehistory_move_'+ css + '">&nbsp;');
            html.push(history[i + 1]);
            html.push('</td>');
        }

        html.push('</tr>');
        css = '';
    }

    $('table.movehistory>tbody').html(html.join(''));
};

ChessClient.prototype.create_timers = function create_timers () {
    var opp_id = '#' + this.config.opp_timer_id,
        own_id = '#' + this.config.own_timer_id,
        self = this;

    if( this.config.opp_timer >= 0 && this.config.own_timer >= 0 ) {
        $(own_id).button();
        $(opp_id).button();

        // set up timers
        client.opponent_timer = $(opp_id).createTimer({
            autostart: false,
            time_in_seconds: Math.floor(self.config.opp_timer / 1000),
            time_format: 'MM:ss',
            buzzer: function(timer){
                self.timeout = true;
                self.check_state();
            }
        });

        client.own_timer = $("#bottom-timer").createTimer({
            autostart: false,
            time_in_seconds: Math.floor(self.config.own_timer / 1000),
            time_format: 'MM:ss',
            buzzer: function(timer){
                self.timeout = true;
                self.check_state();
            }
        });
    }
};
