/**
 * Chess server module
 */

// module dependencies
var u = require('underscore'),
    util = require('util');

/**
 * Class for a chess server
 *
 * This is in charge of handling nodejs chess server features.
 *
 */
function ChessServer(io, config) {
    if (this instanceof ChessServer) {
        // configuration for the nodejs/php server and what would be needed
        this.config = config;
        // socket.io instance
        this.io = io;
        // list of games
        this.games = {};
        // list of players
        this.players = {};
    } else {
        return new ChessServer(io, config);
    }
}

/**
 * Connect player to chess server
 *
 * Executed on every page with real time functionality, except on real time game
 * page (chessrt.php) where `join_player` is executed. On player connect we set
 * the session data and join the online players room. Only using nodejs rooms
 * for broadcasting data, list of players are kept locally.
 *
 * @param player_socket Client of connected player
 * @param session Session data sent for the client connected
 * @param fn Callback
 */
ChessServer.prototype.connect_player = function (player_socket, session, fn) {

    // player identifier will always come in session connect data
    if(typeof session.player !== 'undefined'){
        player_socket.set('player', session.player);
    }

    //find all games in which this player was and notify opponents if any
    this.notify_player_connection(this.games, player_socket, session.player);

    player_socket.join(this.config.players_room);

    if (typeof fn === 'function') fn(player_socket);

    return this;
};

/**
 * Disconnect player from chess server
 *
 * Executed when player reload/leave a page with real time functionality. On
 * player disconnect we set the socket to null and notify online users room and
 * opponents.
 *
 * @param player_socket Client of connected player
 * @param session Session data sent for the client connected
 * @param fn Callback
 */
ChessServer.prototype.disconnect_player = function (player_socket, session, fn) {

    //find all games in which this player was and notify opponents if any
    this.notify_player_disconnection(this.games, player_socket, session.player);

    if (typeof fn === 'function') fn(player_socket);

    return this;
};


/**
 * Notify user connection or reconnection
 *
 * Traverse the list of games looking for the player passed in, if found,
 * notify the opponent, if exist, about its (re)connection.
 *
 * @param player_socket Client of connected player
 * @param playerid Player identifier; user name on mkgichessclub
 */
ChessServer.prototype.notify_player_connection = function notify_player_connection(player_socket, playerid) {
    var player_games = u.filter(this.games, function(game){
        return (typeof game.white !== 'undefined' && game.white.player == playerid) ||
            (typeof game.black !== 'undefined' && game.black.player == playerid);
    });

    u.each(player_games, function(game, gameid){
        // notify opponent if any
        if(typeof game.white !== 'undefined' &&
            game.white.player == playerid) {

            if(typeof game.black.socket !== 'undefined'){
                game.black.socket.emit('opponent connected');
            }
            game.white.socket = player_socket;
        }

        if(typeof game.black !== 'undefined' &&
            game.black.player == playerid) {

            if(typeof game.white.socket !== 'undefined'){
                game.white.socket.emit('opponent connected');
            }
            game.black.socket = player_socket;
        }
    });

    return this;
};

/**
 * Notify user disconnection and delete proper variables
 *
 * Traverse the list of games looking for the player passed in, if found,
 * notify the opponent, if exist, about its disconnection.
 *
 * @param player_socket Client of connected player
 * @param playerid Player identifier; user name on mkgichessclub
 */
ChessServer.prototype.notify_player_disconnection = function notify_player_disconnection(player_socket, playerid) {
    var player_games = u.filter(this.games, function(game){
        return game.white.player == playerid || game.black.player == playerid;
    });

    u.each(player_games, function(game, gameid){
        // notify opponent if any
        if(typeof game.white !== 'undefined' &&
            game.white.player == playerid) {

            if(typeof game.black.socket !== 'undefined'){
                game.black.socket.emit('opponent disconnected');
            }
            // only remove socket
            delete game.white.socket;
        }

        if(typeof game.black !== 'undefined' &&
            game.black.player == playerid) {

            if(typeof game.white.socket !== 'undefined'){
                game.white.socket.emit('opponent disconnected');
            }
            // only remove socket
            delete game.black.socket;
        }
    });

    return this;
};

/**
 * Join a player to a game.
 *
 * When user accept invitation, get redirected to a new url (chessrt.php), its
 * need to connect and also join to the game.
 *
 * @param player_socket Client of connected player
 * @param session Session data sent for the client connected
 * @param fn Callback
 */
ChessServer.prototype.join_player = function (player_socket, session, fn) {
    var self = this;
    // connect player and join him to the game
    self.connect_player(player_socket, session, function(player_socket){
        // now lets make the game stuf
        if(typeof session.game !== 'undefined'){
            player_socket.set('game', session.game);
        } else {
            // error, can not join if not gameid
            console.log('error: joingin game with no gameid');
        }

        if(typeof session.player !== 'undefined'){
            player_socket.set('player', session.player);
        } else {
            // error, can not join if not player
            console.log('error: joingin game with no playerid');
        }

        if(typeof session.color !== 'undefined'){
            player_socket.set('color', session.color);
        } else {
            // error, can not join if not color
            console.log('error: joingin game with no color');
        }

        if(typeof session.invited !== 'undefined'){
            player_socket.set('invited', session.invited);
        } else {
            player_socket.set('invited', 0);
        }

        self.add_player(player_socket, session.game, session.player, session.color);

    });

    if (typeof fn === 'function') fn(player_socket);

    return this;
};

/**
 * Add the player passed in to the game.
 *
 * @param player_socket Client joining to the game.
 * @param gameid Game identifier; integer number on mkgichessclub.
 * @param playerid Player identifier; user name on mkgichessclub
 * @param color Player color in game; `white` or `black`.
 * @private
 */
ChessServer.prototype.add_player = function (player_socket, gameid, playerid, color) {
    if(typeof this.games[gameid] === 'undefined') {
        // create game object
        this.games[gameid] = {white: {gameover: false}, black: {gameover: false}};
    }

    this.games[gameid][color].socket = player_socket;
    this.games[gameid][color].player = playerid;
    this.games[gameid][color].gameover = false;
    // TODO add/create here timer for this player color, timers must be handled
    // server side

    // check if both players are set up to get ready
    if(typeof this.games[gameid].white.socket !== 'undefined' &&
        typeof this.games[gameid].black.socket !== 'undefined') {
        this.games[gameid].white.socket.emit('ready');
        this.games[gameid].black.socket.emit('ready');
        // TODO start timer from here ??
    }

    return this;
};

/**
 * Synchronize players to start timers.
 *
 * This will be removed later once timers are handled by chess server. Timers
 * will start when both clients get ready.
 *
 * @param player_socket socket client sending sync message
 * @param gameid Game identifier
 * @param color color of player
 * @param fn Callback to execute
 */
ChessServer.prototype.sync_players = function sync_players (player_socket, data, fn) {
    // find opponent and send sync
    if(typeof this.games[data.gameid] !== 'undefined') {
        // swap color for opponent one
        var color = (data.color == 'white') ? color = 'black' : color = 'white';

        this.games[data.gameid][color].socket.emit('sync', data, function(synched){
            fn(synched);
        });

    } else {
        // game doesnt exists
        console.log('error: can not sync. game not exist');
        fn('disconnected');
    }
};

/**
 * Invite player to start real time mode game.
 *
 * This will be removed later once timers are handled by chess server. Timers
 * will start when both clients get ready.
 *
 * @param player_socket socket client sending invitation
 * @param playerid Opponent identifier, the receiver
 * @param gameid Game identifier
 * @param time time limit for the game
 * @param fn Callback to execute
 */
ChessServer.prototype.invite_player = function invite_player (player_socket, playerid, gameid, time, fn) {
    var player = u.filter(
            this.io.sockets.clients(this.config.players_room),
            function(opponent_socket){
                var result = '';
                opponent_socket.get('player', function(err, player){
                    result = player;
                });
                return result == playerid;
            }
        ),
        data = {game: gameid, time: time, inviter: ''};

    // find opponent and send invitation
    if(u.size(player) !== 0) {
        player = player[0];

        // get inviter identifier
        player_socket.get('player', function(err, player){
            if (err) throw err;
            data.inviter = player;
        });

        // data.player is the opponent of the one emitting
        player.emit('go real time', data, function(accepted){
            fn(accepted);
        });

    } else {
        // no player found
        fn(false);
    }
};

/**
 * Handle move sent by clients, look for opponent and send data
 *
 * @param player_socket Sockect client making the move.
 * @param data Object Move data information. Containss these properties
 *      move_str:"Pe2-e4",
 *      move_obj:{"src":12,"dest":28,"kill":false},
 *      gameid:5,
 *      player:"davidmpaz",
 *      chess_move:{"from":"e2","to":"e4"},
 *      promotion:false,
 *      own_time:300000,
 *      opp_time:294000
 * @param fn Callback to call
 */
ChessServer.prototype.handle_move = function (player_socket, data, fn) {
    var opponent_socket,
        color = '',
        game = data.gameid;

    player_socket.get("color", function (err, session_color) { color = session_color; });

    if (color == 'white') {
        opponent_socket = this.games[game].black.socket;
    } else {
        opponent_socket = this.games[game].white.socket;
    }

    opponent_socket.emit('update board', data, function(ack){
        // this annonymous function is called on opponent move.
        // calling the callback registered on player moving
        // to notify the opponent player received the move
        fn(ack);
    });
};

/**
 * Hanlde timeout event from clients. This si executed when clients run out
 * of time and a game over by timeout event is sent.
 *
 * @param data Object
 * @param fn Function callback
 */
ChessServer.prototype.handle_timeout = function handle_timeout(player_socket, data, fn){
    var self = this,
        color = '',
        gameid = data.gameid;

    player_socket.get("color", function(err, session_color){ color = session_color; });

    if (color == 'white') {
        if(typeof self.games[gameid].black.socket !== 'undefined') {
            self.games[gameid].black.socket.emit('game over timeout', data, function(ack){
                if(ack == 'ok'){
                    self.games[gameid].black.gameover = true;
                }
                fn(ack);
            });
        }
        self.games[gameid].white.gameover = true;
    } else {
        if(typeof self.games[gameid].white.socket !== 'undefined') {
            self.games[gameid].white.socket.emit('game over timeout', data, function(ack){
                if(ack == 'ok'){
                    self.games[gameid].white.gameover = true;
                }
                fn(ack);
            });
        }

        self.games[gameid].black.gameover = true;
    }

    if(typeof self.games[gameid].black.gameover != 'undefined' &&
        self.games[gameid].white.gameover != 'undefined' &&
        self.games[gameid].black.gameover &&
        self.games[gameid].white.gameover) {
        // both ends acknowledged, we are done
        self.games[gameid].black.socket.emit('finish');
        self.games[gameid].white.socket.emit('finish');
    }
};

/**
 * Makes a request to php server to get the corresponding html for the online
 * player list show in home page. This will do the same job as the current online
 * player functionality, a burden presence functionality to get a clue of which
 * are playing chess at that moment.
 */
ChessServer.prototype.get_online_players = function () {
    var mysql = require('mysql'),
        result = false,
        self = this;
    var connection = mysql.createConnection({
        host     : this.config.dbhost,      // localhost
        user     : this.config.dbuser,      // root
        password : this.config.dbpassword,  // ok
        database : this.config.database     // chess
    });

    connection.connect();

    connection.query("select pl_identifier, pl_score_points " +
        "from mcc_player " +
        "where pl_is_active = 1 " +
        "   and pl_is_validated = 1 " +
        "   and pl_email_address is not null " +
        "   and ( unix_timestamp( now() ) - unix_timestamp(pl_last_check) ) <= 60000 / 1000 " +
        "order by pl_identifier desc", function(err, rows, fields) {

        if (err) throw err;

        // push online players to clients
        self.io.sockets.in(self.config.players_room).emit('update online players', rows);

    });

    connection.end();
};



// export our Chess server class
module.exports.create = function (io, config) {
    return new ChessServer(io, config);
};
